@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <https://www.gnu.org/licenses/>.
 
-#pylint: disable=wrong-import-position
+# pylint: disable=wrong-import-position
 import logging
 import gi
 gi.require_version('Gst', '1.0')
@@ -30,7 +30,8 @@ class gTranscribePlayer(Gst.Bin):
 
     __gsignals__ = {
         'ready': (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_STRING,)),
-        'ended': (GObject.SignalFlags.RUN_LAST, None, ())
+        'ended': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'duration_changed': (GObject.SignalFlags.RUN_LAST, None, ())
     }
 
     def __init__(self):
@@ -38,6 +39,7 @@ class gTranscribePlayer(Gst.Bin):
 
         self._rate = 1
         self._duration = None
+        self._messageType = Gst.MessageType.UNKNOWN
         self.init_pipeline()
 
     def init_pipeline(self):
@@ -88,9 +90,13 @@ class gTranscribePlayer(Gst.Bin):
     @property
     def duration(self):
         """Return the duration of the current stream."""
-        if self._duration is None:
-            self._duration = self.pipeline.query_duration(Gst.Format.TIME)[1]
-            logger.debug('Duration is: "%s"', self._duration)
+        success = False
+        if self._duration is None or self._messageType == Gst.MessageType.DURATION_CHANGED:
+            success, self._duration = self.pipeline.query_duration(Gst.Format.TIME)
+            if not success:
+                self._duration = 0
+            logger.debug('Query was successful: %s, Duration is: "%s"',
+                         success, self._duration)
         return self._duration
 
     def _get_position(self):
@@ -128,7 +134,6 @@ class gTranscribePlayer(Gst.Bin):
         # Position query was not successful
         if pos[0] == 0:
             seek_type = Gst.SeekType.NONE
-            pos[1] = -1
         self.pipeline.seek(rate,
                            Gst.Format.TIME,
                            Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
@@ -175,12 +180,15 @@ class gTranscribePlayer(Gst.Bin):
     # pylint: disable=unused-argument
     def on_message(self, bus, message):
         """Handle message and react accordingly."""
+        self._messageType = message.type
         if message.type == Gst.MessageType.EOS:
             self.state = Gst.State.NULL
             self.emit('ended')
         elif message.type == Gst.MessageType.ERROR:
             logger.debug("%s", print(message.parse_error()))
             self.state = Gst.State.NULL
+        elif message.type == Gst.MessageType.DURATION_CHANGED:
+            self.emit('duration_changed')
 
     def open(self, filepath, duration=True):
         """Open audio file and optionally query duration."""
